datacache <- new.env(hash=TRUE, parent=emptyenv())

gahgu95e <- function() showQCData("gahgu95e", datacache)
gahgu95e_dbconn <- function() dbconn(datacache)
gahgu95e_dbfile <- function() dbfile(datacache)
gahgu95e_dbschema <- function(file="", show.indices=FALSE) dbschema(datacache, file=file, show.indices=show.indices)
gahgu95e_dbInfo <- function() dbInfo(datacache)

gahgu95eORGANISM <- "Homo sapiens"




GENECARDS_DB_L2Rlink1 <- list(tablename="probes", Lcolname="probe_id", Rcolname="_id")

### Mandatory fields: objName, Class and L2Rchain
GENECARDS_DB_AnnDbBimap_seeds <- list(
    list(
        objName="ENTREZID",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="entrezgene",
                Lcolname="_id",
                Rcolname="entrezgene_id"
            )
        )
    ),
    list(
        objName="GENECARDSID",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="genes",
                Lcolname="_id",
                Rcolname="gene_id"
            )
        )
    ),
    list(
        objName="GENENAME",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="gene_info",
                Lcolname="_id",
                Rcolname="gene_name"
            )
        )
    ),
    list(
        objName="SYMBOL",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="gene_info",
                Lcolname="_id",
                Rcolname="symbol"
            )
        )
    ),
    list(
        objName="ALIAS2PROBE",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="alias",
                Lcolname="_id",
                Rcolname="alias_symbol"
            )
        ),
        direction=-1L
    ),
    list(
        objName="ALIAS",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="alias",
                Lcolname="_id",
                Rcolname="alias_symbol"
            )
        )
    ),
    list(
        objName="DESCRIPTIONS",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="descriptions",
                Lcolname="_id",
                tagname=c(description_source="{description_source}"),
                Rcolname="description"
            )
        )
    ),
    list(
        objName="CHR",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="chromosomes",
                Lcolname="_id",
                Rcolname="chromosome"
            )
        )
    ),
    list(
        objName="CHRLOC",
        Class="AnnDbMap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="chromosome_locations",
                Lcolname="_id",
                tagname=c(Chromosome="{seqname}"),
                Rcolname="start_location"
            )
        )
#        rightColType="integer"
    ),
    list(
        objName="CHRLOCEND",
        Class="AnnDbMap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="chromosome_locations",
                Lcolname="_id",
                tagname=c(Chromosome="{seqname}"),
                Rcolname="end_location"
            )
        )
#        rightColType="integer"
    ),
    list(
        objName="GENESTART",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="genestart",
                Lcolname="_id",
                Rcolname="gene_start"
            )
        )
#        rightColType="integer"
    ),
    list(
        objName="GENEEND",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="geneend",
                Lcolname="_id",
                Rcolname="gene_end"
            )
        )
#        rightColType="integer"
    ),
    list(
        objName="GENESTRAND",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="strands",
                Lcolname="_id",
                Rcolname="strand"
            )
        )
    ),
    list(
        objName="INTERPROID",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="interpro",
                Lcolname="_id",
                Rcolname="interpro_id"
            )
        )
    ),
    list(
        objName="INTERPRODOMAIN",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="interpro",
                Lcolname="_id",
                Rcolname="interpro_domain",
                tagname=c(interpro_id="{interpro_id}")
            )
        )
    ),
   list(
        objName="UNIPROT",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="uniprot",
                Lcolname="_id",
                Rcolname="uniprot_information",
                tagname=c(uniprot_information_type="{uniprot_information_type}")
            )
        )
    ),
    list(
        objName="SWISSPROTID",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="swissprot",
                Lcolname="_id",
                Rcolname="swissprot_id"
            )
        )
    ),
    list(
        objName="KEGGID",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="kegg",
                Lcolname="_id",
                Rcolname="path_id"
            )
        )
    ),
    list(
        objName="KEGGNAME",
        Class="AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                tablename="kegg",
                Lcolname="_id",
                Rcolname="path_name",
                tagname=c(path_id="{path_id}")
            )
        )
    ),
    list(
        objName="GO",
        Class="Go3AnnDbBimap",
        L2Rchain=list(
            GENECARDS_DB_L2Rlink1,
            list(
                #tablename="go_term", # no rightmost table for a Go3AnnDbBimap
                Lcolname="_id",
                tagname=c(Evidence="{evidence}"),
                Rcolname="go_id",
                Rattribnames=c(Ontology="NULL")
            )
        ),
        rightTables=AnnotationDbi:::Go3tablenames()
    )
)

createAnnObjs.GENECARDS_DB <- function(prefix, objTarget, dbconn, datacache)
{
    ## checkDBSCHEMA(dbconn, "GENECARDS_DB")

    ## AnnDbBimap objects
    seed0 <- list(
        objTarget=objTarget,
        datacache=datacache
    )
    ann_objs <- AnnotationDbi:::createAnnDbBimaps(GENECARDS_DB_AnnDbBimap_seeds, seed0)

#     ## Reverse maps
#     ann_objs$ACCNUM2EG <- revmap(ann_objs$ACCNUM, objName="ACCNUM2EG")
#     ann_objs$ENZYME2EG <- revmap(ann_objs$ENZYME, objName="ENZYME2EG")
#     ann_objs$MAP2EG <- revmap(ann_objs$MAP, objName="MAP2EG")
#     ann_objs$OMIM2EG <- revmap(ann_objs$OMIM, objName="OMIM2EG")
#     ann_objs$PATH2EG <- revmap(ann_objs$PATH, objName="PATH2EG")
#     ann_objs$PMID2EG <- revmap(ann_objs$PMID, objName="PMID2EG")
#     ann_objs$REFSEQ2EG <- revmap(ann_objs$REFSEQ, objName="REFSEQ2EG")
     ann_objs$ENTREZID2PROBE <- AnnotationDbi:::revmap(ann_objs$ENTREZID, objName="ENTREZID2PROBE")
     ann_objs$SYMBOL2PROBE <- AnnotationDbi:::revmap(ann_objs$SYMBOL, objName="SYMBOL2PROBE")
#     ann_objs$UNIGENE2EG <- revmap(ann_objs$UNIGENE, objName="UNIGENE2EG")
#     #ann_objs$PFAM2EG <- revmap(ann_objs$PFAM, objName="PFAM2EG")
#     #ann_objs$PROSITE2EG <- revmap(ann_objs$PROSITE, objName="PROSITE2EG")
#     ann_objs$ENSEMBL2EG <- revmap(ann_objs$ENSEMBL, objName="ENSEMBL2EG")
#     ann_objs$ENSEMBLPROT2EG <- revmap(ann_objs$ENSEMBLPROT, objName="ENSEMBLPROT2EG")
#     ann_objs$ENSEMBLTRANS2EG <- revmap(ann_objs$ENSEMBLTRANS, objName="ENSEMBLTRANS2EG")
    ann_objs$GO2PROBE <- AnnotationDbi:::revmap(ann_objs$GO, objName="GO2PROBE")

#     map <- ann_objs$GO2PROBE
#     map@rightTables <- Go3tablenames(all=TRUE)
#     map@objName <- "GO2ALLPROBES"
#     ann_objs$GO2ALLPROBES <- map

    ## 2 special maps that are not AnnDbBimap objects (just named integer vectors)
#    ann_objs$CHRLENGTHS <- createCHRLENGTHS(dbconn)
    ann_objs$MAPCOUNTS <- AnnotationDbi:::createMAPCOUNTS(dbconn, prefix)

    ## Some pre-caching
    AnnotationDbi:::Lkeys(ann_objs$GO)
    #mappedLkeys(ann_objs$GO)
    #Rkeys(ann_objs$GO2EG)
    #mappedRkeys(ann_objs$GO2EG)

    AnnotationDbi:::prefixAnnObjNames(ann_objs, prefix)
}


.onLoad <- function(libname, pkgname)
{
    require("methods", quietly=TRUE)
    ## Connect to the SQLite DB
    dbfile <- system.file("extdata", "gahgu95e.sqlite", package=pkgname, lib.loc=libname)
    assign("dbfile", dbfile, envir=datacache)
    dbconn <- dbFileConnect(dbfile)
    assign("dbconn", dbconn, envir=datacache)
    ## Create the AnnObj instances
    ann_objs <- createAnnObjs.GENECARDS_DB("gahgu95e", "Geneannot (Human)", dbconn, datacache)
    mergeToNamespaceAndExport(ann_objs, pkgname)
}

.onUnload <- function(libpath)
{
    dbFileDisconnect(gahgu95e_dbconn())
}


